﻿/*
    
    Analyzes tif as text files prepared during a previously run analysis. Requires both the property file and the tif to text file for each sample analyzed.
    
*/

// set the debug level appropriately, remark if editing code
//$.level = 0;

//  set the target application to photoshop
#target photoshop

//  include the BubbleCounter script or none of this beautiful work will
#include "BC_Classes/BubbleCounter.jsxinc"
#include "BC_Classes/BC_Defaults.jsxinc"

//  turn off dialogs, seems to speed processing?
app.displayDialogs = DialogModes.NO;
//  get the present ruler units so that the app can be set back to its original state
var uStrtRulerUnits = app.preferences.rulerUnits;
//  set the ruler units to pixels
app.preferences.rulerUnits = Units.PIXELS;

// following set by user to tell BC what to do...
var createReport = false; // if set to true, all the BC reports in the selected folder will be overwritten by the results from this run
var createRawReport = false; // set this to write the raw void chord length report


/*
    choices for the following:
        "default" = what it finds in BC defaults
        "props" = what it finds in the image's property file
        values you enter

*/
var thresholdAirContent = 97;
var thresholdVoidFreq = 109;
var airCorrectionFactor = "props";

var analysisMethod = 4; // similar to above, but isn't found in BC defaults, so "default" isn't an option

var chordLengthCutoff = 0; // chords longer than this, in microns, will be discarded from the analysis;
                                              // the pixels will not be used in the numerator or denominator of the air content calculation and chords will not be used for void frequency calculation
                                              // if set to zero, all chord lengths will be used in calculations
                                              // This is an attempt to eliminate entrapped air in poorly consolidated samples
var bc;
var sPath = new Folder().selectDlg ("Folder containing prepared images");
var propsLoaded = false;

if (sPath != null && sPath != undefined)
    {
        var asFiles = new Folder(sPath).getFiles("*.txt");
        var countFiles = asFiles.length;
        var countAnalyzed = 0;
        var aFileNameOriginalImage = new Array();
        var aAnalysisMethod = new Array();
        var aTraverseLengthmm = new Array();
        var aAirLengthmm = new Array();
        var aIntercepts = new Array();
        var aChordLengthAveragemm = new Array();
        var aSpecificSurface = new Array();
        var aVoidFreq = new Array();
        var aAirContent = new Array();
        var aSpacingFactor = new Array();
        var aThresholdVF = new Array();
        var aThresholdAC = new Array();
        // following added for capturing data after applying the filter - F stands for Filter
        var aFTraverseLengthmm = new Array();
        var aFAirLengthmm = new Array();
        var aFIntercepts = new Array();
        var aFChordLengthAveragemm = new Array();
        var aFSpecificSurface = new Array();
        var aFVoidFreq = new Array();
        var aFAirContent = new Array();
        var aFSpacingFactor = new Array();
        var aFThresholdVF = new Array();
        var aFThresholdAC = new Array();

        if (countFiles > 0)
            {
                var timeStart = new Date();
                var fImage;
                //  iterate through the list and analyze each
                for (var i = 0; i < countFiles; i++)
                    {
                                countAnalyzed++;
                                bc = new BubbleCounter();
                                bc.path = sPath;
                                bc.fileNameOriginalImage = asFiles[i].name.replace("_comp.txt", ".tif");
                                bc.fileNameCompositeImage = bc.fileNameOriginalImage.replace(".tif", "_comp.tif");
                                bc.fileNameTabDelimited = bc.fileNameOriginalImage.replace(".tif", "_comp.txt");
                                bc.fileNameProperties = bc.fileNameOriginalImage.replace(".tif", "_props.csv");
                                bc.fileProperties = new PropertyFile(bc.path, bc.fileNameProperties);
                                bc.createReport = createReport;
                                bc.createRawReport = createRawReport;
                                //  look for the property file
                                if (bc.readPropertyFile())
                                    {
                                        //  look for traverse bounds to determine if the analysis has been run
                                        if (bc.fileProperties.boundsTraverses.length ==  parseInt(bc.fileProperties.traverses))
                                            {
                                                 switch (thresholdAirContent)
                                                    {
                                                        case "props" :
                                                            bc.thresholdAirContent = bc.fileProperties.thresholdAirContent;
                                                            break;
                                                        case "default" :
                                                            bc.thresholdAirContent = bcThresholdAirContent;
                                                            break;
                                                        default :
                                                            if (isNaN(thresholdAirContent))
                                                                {
                                                                    alert("thresholdAirContent must be numeric, unable to continue...");
                                                                    i = 10000;
                                                                }
                                                            else
                                                                {
                                                                    bc.thresholdAirContent = thresholdAirContent;
                                                                }
                                                            break;
                                                    }
                                                 switch (thresholdVoidFreq)
                                                    {
                                                        case "props" :
                                                            bc.thresholdVoidFreq = bc.fileProperties.thresholdVoidFreq;
                                                            break;
                                                        case "default" :
                                                            bc.thresholdVoidFreq = bcThresholdVoidFreq;
                                                            break;
                                                        default :
                                                            if (isNaN(thresholdVoidFreq))
                                                                {
                                                                    alert("thresholdVoidFreq must be numeric, unable to continue...");
                                                                    i = 10000;
                                                                }
                                                            else
                                                                {
                                                                    bc.thresholdVoidFreq = thresholdVoidFreq;
                                                                }
                                                            break;
                                                    }
                                                 switch (airCorrectionFactor)
                                                    {
                                                        case "props" :
                                                            bc.airCorrectionFactor = bc.fileProperties.airCorrectionFactor;
                                                            break;
                                                        case "default" :
                                                            bc.airCorrectionFactor = bcAirCorrection;
                                                            break;
                                                        default :
                                                            if (isNaN(airCorrectionFactor))
                                                                {
                                                                    alert("airCorrectionFactor must be numeric, unable to continue...");
                                                                    i = 10000;
                                                                }
                                                            else
                                                                {
                                                                    bc.airCorrectionFactor = airCorrectionFactor;
                                                                }
                                                            break;
                                                    }
                                                 switch (analysisMethod)
                                                    {
                                                        case "props" :
                                                            bc.analysisMethod = bc.fileProperties.analysisMethod;
                                                            break;
                                                        default :
                                                            if (isNaN(analysisMethod))
                                                                {
                                                                    alert("analysisMethod must be numeric, unable to continue...");
                                                                    i = 10000;
                                                                }
                                                            else
                                                                {
                                                                    bc.analysisMethod = analysisMethod;
                                                                }
                                                            break;
                                                    }
                                                bc.micronsPerPixel = 25400 / bc.resolution;
                                                bc.readCompositeText();
                                                bc.processTraversesVF();
                                                bc.processTraversesAC();
                                                bc.processTraversesAvg();
                                                bc.applyAirCorrection();
                                                bc.calculate();
                                                bc.generateReport();
                                                bc.generateRawReport();
                                                aFileNameOriginalImage[i] = bc.fileNameOriginalImage;
                                                aAnalysisMethod[i] = bc.analysisMethod;
                                                aTraverseLengthmm[i] = bc.traverseLengthmm;
                                                aAirLengthmm[i] = bc.airLengthmm;
                                                aIntercepts[i] = bc.intercepts;
                                                aChordLengthAveragemm[i] = bc.chordLengthAveragemm;
                                                aSpecificSurface[i] = bc.specificSurface;
                                                aVoidFreq[i] = bc.voidFreq;
                                                aAirContent[i] = bc.airContent;
                                                aSpacingFactor[i] = bc.spacingFactor;
                                                aThresholdVF[i] = bc.thresholdVoidFreq;
                                                aThresholdAC[i] = bc.thresholdAirContent;
                                                if (chordLengthCutoff > 0)
                                                    {
                                                        // do it all again, but don't count chords longer than the cutoff
                                                        var fPixelsAir = 0;
                                                        var fPixelsAnalyzed = bc.pixelsAnalyzed;
                                                        var fIntercepts = bc.intercepts;
                                                        // if analysis method 0, 1, 2 or 3 is used, then just use bc.chordsTotal since only a single threshold was applied
                                                        // if analysis method 4 is used, then need to reprocess traverses
                                                        if (bc.analysisMethod == 4 )
                                                            {
                                                                var processedTraversesVF = bc.processTraverses(bc.thresholdVoidFreq);
                                                                var processedTraversesAC = bc.processTraverses(bc.thresholdAirContent);
                                                                // throw out chords larger than the cutoff from VF optimized to calculate filtered intercepts
                                                                for (var j = 0; j < processedTraversesVF.chordsTotal.chordCount; j++)
                                                                    {
                                                                        if (processedTraversesVF.chordsTotal.chords[j].pixels * bc.micronsPerPixel > chordLengthCutoff)
                                                                            {
                                                                                fIntercepts -= processedTraversesVF.chordsTotal.chords[j].count;
                                                                            }
                                                                    }
                                                                // now fix the air content
                                                                for (var j = 0; j < processedTraversesAC.chordsTotal.chordCount; j++)
                                                                    {
                                                                        if (processedTraversesAC.chordsTotal.chords[j].pixels * bc.micronsPerPixel > chordLengthCutoff)
                                                                            {
                                                                                fPixelsAnalyzed -= processedTraversesAC.chordsTotal.chords[j].pixels * processedTraversesAC.chordsTotal.chords[j].count;
                                                                            }
                                                                        else
                                                                            {
                                                                                fPixelsAir += processedTraversesAC.chordsTotal.chords[j].pixels * processedTraversesAC.chordsTotal.chords[j].count;
                                                                            }
                                                                    }
                                                            }
                                                        else
                                                            {
                                                                var chordsTotal;
                                                                switch (bc.analysisMethod)
                                                                    {
                                                                        case 0 :
                                                                            chordsTotal = bc.chordsTotalCorrected;
                                                                            break;
                                                                        case 1 :
                                                                            chordsTotal = bc.chordsTotalVF;
                                                                            break;
                                                                        case 2 :
                                                                            chordsTotal = bc.chordsTotalAC;
                                                                            break;
                                                                        default :
                                                                            chordsTotal = bc.chordsTotalAvg;
                                                                            break;
                                                                    }
                                                                for (var j = 0; j < chordsTotal.chordCount; j++)
                                                                    {
                                                                        if (chordsTotal.chords[j].pixels * bc.micronsPerPixel > chordLengthCutoff)
                                                                            {
                                                                                fPixelsAnalyzed -= chordsTotal.chords[j].pixels * chordsTotal.chords[j].count;
                                                                            }
                                                                        else
                                                                            {
                                                                                fPixelsAir += chordsTotal.chords[j].pixels * chordsTotal.chords[j].count;
                                                                                fIntercepts += chordsTotal.chords[j].count;
                                                                            }
                                                                    }
                                                                
                                                            }
                                                        
                                                        bc.pixelsAir = fPixelsAir;
                                                        bc.pixelsAnalyzed = fPixelsAnalyzed;
                                                        bc.intercepts = fIntercepts;
                                                        bc.calculate();
                                                        aFTraverseLengthmm[i] = bc.traverseLengthmm;
                                                        aFAirLengthmm[i] = bc.airLengthmm;
                                                        aFIntercepts[i] = bc.intercepts;
                                                        aFChordLengthAveragemm[i] = bc.chordLengthAveragemm;
                                                        aFSpecificSurface[i] = bc.specificSurface;
                                                        aFVoidFreq[i] = bc.voidFreq;
                                                        aFAirContent[i] = bc.airContent;
                                                        aFSpacingFactor[i] = bc.spacingFactor;
                                                        aFThresholdVF[i] = bc.thresholdVoidFreq;
                                                        aFThresholdAC[i] = bc.thresholdAirContent;
                                                    }
                                            }
                                    }
                                else
                                    {
                                        // we're done -  no prop file, no analysis
                                    }
                    }
                var timeEnd = new Date();
            }
            var fResults = new File(sPath + "/" + "resultsummary.csv");
            with (fResults)
                {
                    open("w");
                    writeln("Analysis of," + aFileNameOriginalImage.join(","));
                    writeln("Analysis method," + aAnalysisMethod.join(","));
                    writeln("Applied thresholdVF," + aThresholdVF.join(","));
                    writeln("Applied thresholdAC," + aThresholdAC.join(","));
                    writeln("Total length of traverse (mm)," + aTraverseLengthmm.join(","));
                    writeln("Traverse length through air (mm)," + aAirLengthmm.join(","));
                    writeln("Total number of air voids intercepted," +aIntercepts.join(","));
                    writeln("Average chord length (mm)," + aChordLengthAveragemm.join(","));
                    writeln("Specific surface (mm-1)," + aSpecificSurface.join(","));
                    writeln("Void frequency (intercepts/mm)," + aVoidFreq.join(","));
                    writeln("Air content %," + aAirContent.join(","));
                    writeln("Spacing Factor (mm)," + aSpacingFactor.join(","));
                    if (chordLengthCutoff > 0)
                        {
                            writeln();
                            writeln();
                            writeln("Results after application of chord length cutoff of " + chordLengthCutoff + " microns:");
                            writeln("Total length of traverse (mm)," + aFTraverseLengthmm.join(","));
                            writeln("Traverse length through air (mm)," + aFAirLengthmm.join(","));
                            writeln("Total number of air voids intercepted," +aFIntercepts.join(","));
                            writeln("Average chord length (mm)," + aFChordLengthAveragemm.join(","));
                            writeln("Specific surface (mm-1)," + aFSpecificSurface.join(","));
                            writeln("Void frequency (intercepts/mm)," + aFVoidFreq.join(","));
                            writeln("Air content %," + aFAirContent.join(","));
                            writeln("Spacing Factor (mm)," + aFSpacingFactor.join(","));
                        }
                    close();
                }
        
        var msgAlert = "Finished analyzing " + countAnalyzed + " images.\n\A summary containing results has been written in the folder selected.";
        if (createReport) msgAlert += "\n\Detailed reports were written for each image analyzed.";
        if (createRawReport) msgAlert += "\n\Chord length data were written for each image analyzed.";
        
        alert(msgAlert);
    }
app.preferences.rulerUnits = uStrtRulerUnits;

