﻿/*
    
    Analyze a folder of images that have been cropped and stretched
    
*/

// set the debug level appropriately, remark if editing code
//$.level = 0;

//  set the target application to photoshop
#target photoshop

//  include the BubbleCounter script or none of this beautiful work will
#include "BC_Classes/BubbleCounter.jsxinc"
#include "BC_Classes/BC_Defaults.jsxinc"

//  turn off dialogs, seems to speed processing?
app.displayDialogs = DialogModes.NO;
//  get the present ruler units so that the app can be set back to its original state
var uStrtRulerUnits = app.preferences.rulerUnits;
//  set the ruler units to pixels
app.preferences.rulerUnits = Units.PIXELS;

var bc;
var sPath = new Folder().selectDlg ("Folder containing prepared images");
var propsLoaded = false;
if (sPath != null && sPath != undefined)
    {
        var asFiles = new Folder(sPath).getFiles("*.tif");
        var countFiles = asFiles.length;
        var countAnalyzed = 0;
        if (countFiles > 0)
            {
                var timeStart = new Date();
                var fImage;
                var fResults = new File(sPath + "/" + "resultsummary.csv");
                with (fResults)
                    {
                        open("w");
                        writeln("Summary of Bubble Counter analyses");
                        writeln("Folder," + sPath);
                        writeln("Threshold," + bcThreshold);
                        writeln("Air correction factor," + bcAirCorrection);
                        writeln("Started," + timeStart);
                        writeln("File,Air Content,Void Frequency");
                        close();
                    }
                //  iterate through the list and analyze each
                for (var i = 0; i < countFiles; i++)
                    {
                        //  don't analyze _comp.tif files
                        if (asFiles[i].name.match(/_comp.tif/gi) == null)
                            {
                                countAnalyzed++;
                                bc = new BubbleCounter();
                                bc.createRawReport = true;
                                bc.path = sPath;
                                bc.fileNameOriginalImage = asFiles[i].name;
                                bc.fileNameCompositeImage = bc.fileNameOriginalImage.replace(".tif", "_comp.tif");
                                bc.fileNameTabDelimited = bc.fileNameOriginalImage.replace(".tif", "_comp.txt");
                                bc.fileNameProperties = bc.fileNameOriginalImage.replace(".tif", "_props.csv");
                                bc.fileProperties = new PropertyFile(bc.path, bc.fileNameProperties);
                                var reloaded = false;
                                //  look for the property file
                                if (bc.readPropertyFile())
                                    {
                                        //  see if there are already traverse bounds
                                        if (bc.fileProperties.boundsTraverses.length ==  parseInt(bc.fileProperties.traverses))
                                            {
//                                                var aBounds = [bc.fileProperties.boundsTraverses[0][0], bc.fileProperties.boundsTraverses[0][3], bc.fileProperties.boundsTraverses[bc.fileProperties.traverses - 1][3], bc.fileProperties.boundsTraverses[bc.fileProperties.traverses - 1][0]];
//                                                bc.documentOriginal.selection.select(aBounds);
                                                reloaded = true;
                                                propsLoaded = true;
                                            }
                                    }
                                else
                                    {
                                        //  look for the last settings file and load it up
                                        propsLoaded = bc.readLastSettings();
                                    }
                                if (propsLoaded)
                                    {
                                        if (!reloaded)
                                            {                                                
                                                app.open(asFiles[i]);
                                                app.activeDocument.selection.selectAll();
                                                bc.documentOriginal = app.activeDocument;
                                                bc.boundsAnalysis = bc.documentOriginal.selection.bounds;
                                                bc.resolution = bc.documentOriginal.resolution;
                                                bc.createTraverseComposite();
                                                bc.closeDocumentNoSave(bc.documentOriginal);
                                                bc.saveDocumentAsTif(bc.documentComposite, bc.fileNameCompositeImage);
                                                bc.saveDocumentAsText(bc.documentComposite, bc.fileNameTabDelimited);
                                                bc.closeDocumentNoSave(bc.documentComposite);
                                            }
                                        bc.micronsPerPixel = 25400 / bc.resolution;
                                        bc.threshold = bcThreshold;
                                        bc.airCorrectionFactor = bcAirCorrection;
                                        bc.processTraverses();
                                        bc.calculate(true);
                                        bc.generateReport();
                                        bc.generateRawReport();
                                        bc.writePropertyFile();
                                        //  write the results to a file
                                        with (fResults)
                                            {
                                                open("a");
                                                writeln(bc.fileNameOriginalImage + "," + bc.airContent + "," + bc.voidFreq);
                                                close();
                                            }
                                    }
                                else
                                    {
                                        alert("Unable to set properties.  Insure bc_settings.csv exists in the folder.\n Failed:\n" + asFiles[i].name);
                                    }
                            }
                    }
                var timeEnd = new Date();
                with (fResults)
                    {
                        open("a");
                        writeln("Files Analyzed," + countAnalyzed);
                        writeln("Finished," + timeEnd);
                        writeln("Total time (s)," + (timeEnd - timeStart) / 1000);
                        close();
                    }
            }
        
    }
alert("Finished analyzing " + countAnalyzed + " images.\n\Detailed reports as well as a summary of results may be found in the selected folder.");
app.preferences.rulerUnits = uStrtRulerUnits;

