﻿/*
    Since there can be no interaction with Photoshop once a script is running, the user must make a selection before the script
    is fired.  The BubbleCounter class will look to see if there is a selection and execution will halt if there is not.
    
    The image being analyzed must first have the white balance values determined by running BC Set White Balance.
    If this isn't done, the script will halt execution and alert the user.
    
    User selects an area of the image that will be analyzed and then fires this script.  User will be prompted for necessary
    and additional data prior to running the analysis and will be given choices as to reporting.  Data validation should capture
    any button pushing monkey action before running the analysis.
    
    NOTE:  Performance is greatly enhanced by minimizing the Photoshop application after clicking the Analyze button.
    If the user chooses to be notified with the analysis summary, a neat little bell will sound and an alert box with pertinent
    analysis results will be displayed.
    
*/

// set the debug level appropriately, remark if editing code
$.level = 0;

//  set the target application to photoshop
#target photoshop

//  include the BubbleCounter script or none of this beautiful work will
#include "BC_Classes/BubbleCounter.jsxinc"

//  turn off dialogs, seems to speed processing?
app.displayDialogs = DialogModes.NO;
//  get the present ruler units so that the app can be set back to its original state
var uStrtRulerUnits = app.preferences.rulerUnits;
//  set the ruler units to pixels
app.preferences.rulerUnits = Units.PIXELS;

// make sure a document is open
if (app.documents.length > 0)
    {
        //  instantiate a BubbleCounter class
        var bc = new BubbleCounter();
        // setup bc
        bc.createRawReport = false;
        bc.suppressWarnings = true;
        //  set the documentOriginal property to the currently active document
        bc.documentOriginal = app.activeDocument;
         // make sure that something has been selected
        if (bc.isSelectionMade())
            {
                // probably way to much encapsulated by BC, but it makes this part of the script pretty...
                bc.runAnalysis();
            }
        else
            {
                alert("Please select an area of the slab to analyze and rerun this script.");
            }
    }
 else
    {
        alert("Please open an image of a scanned, prepared slab, run the stretch script, select an area to analyze and rerun this script.")
    }

app.preferences.rulerUnits = uStrtRulerUnits;