﻿/*
*******************************    
    
    Open all of the images in a folder, apply stretch, apply different pixel dimensions (resolution) and save as a pub-ready jpg file.
    This applies changes to entire image, not just the analysis area.
    
    
*******************************    
*/

//  set the target application to photoshop
#target photoshop

//  include the BubbleCounter script or none of this beautiful work will
#include "BC_Classes/BubbleCounter.jsxinc"
#include "BC_Classes/PropertyFile.jsxinc"

// set the following to the values you want
var newResolution = 600; // pixels per inch to resize the selected area to
var nameFolderPubReady = "/pub ready02/"; // folder name where jpgs will be saved. If subsequent runs are made without changing this, contents will be overwritten

var sPath = new Folder().selectDlg ("Folder containing processed images");

if (sPath != undefined)
    {
        // setup the results folder
        var folderPubReady = new Folder(sPath + nameFolderPubReady);
        if (!folderPubReady.exists) folderPubReady.create();

        //  get all the property files in the folder
        var asFiles = new Folder(sPath).getFiles("*_props.csv");
        var countPropertyFiles = asFiles.length;
        var pf;
        var df;
        var bc;
        for (var i = 0; i < countPropertyFiles; i++)
            {
                pf = new PropertyFile(asFiles[i].path, asFiles[i].name);
                pf.read();
                df = File(sPath + "/" + pf.fileNameOriginalImage);
               // make sure we have everything we need
                var fileNameOriginalImage = pf.fileNameOriginalImage;
                var boundsAnalysis = pf.boundsAnalysis;
                var resolution = pf.resolution;
                var intensityBlackMax = pf.intensityBlackMax;
                var intensityWhiteMax = pf.intensityWhiteMax;
                var stepHorizontal = pf.stepHorizontal;
                
                var exception = "";
                if (fileNameOriginalImage == undefined || fileNameOriginalImage == "undefined") exception += "no fileNameOriginalImage\n";
                if (boundsAnalysis == undefined || boundsAnalysis == "undefined") exception += "no boundsAnalysis\n";
                if (intensityBlackMax == undefined || intensityBlackMax == "undefined") exception += "no intensityBlackMax\n";
                if (intensityWhiteMax == undefined || intensityWhiteMax == "undefined") exception += "no intensityWhiteMax\n";
                if (stepHorizontal == undefined || stepHorizontal == "undefined") exception += "no stepHorizontal\n";
                if (!df.exists) exception += "Original image file not found\n";
                
                if (exception == "")
                    {
                        var fileNamePubReadyImage = fileNameOriginalImage.replace(".tif", ".jpg");
                        open(df);
                        // instantiate a new BC
                        bc = new BubbleCounter(); 
                        // setup BC
                        bc.documentOriginal = app.activeDocument;
                        bc.intensityBlackMax = intensityBlackMax;
                        bc.intensityWhiteMax = intensityWhiteMax;
                        bc.setDocumentLevels(bc.documentOriginal);
                        bc.documentOriginal.selection.selectAll()
                        bc.documentOriginal.selection.copy();
                        var documentSelection = app.documents.add(bc.documentOriginal.width, bc.documentOriginal.height, 600, fileNamePubReadyImage, NewDocumentMode.GRAYSCALE, DocumentFill.TRANSPARENT, 1.0, BitsPerChannelType.EIGHT);
                        documentSelection.resizeImage(undefined, undefined, resolution, ResampleMethod.NONE); 
                        app.activeDocument = documentSelection;
                        documentSelection.paste();
                        documentSelection.resizeImage(undefined, undefined, newResolution, ResampleMethod.BICUBICSHARPER);
                        var jpgOpt = new JPEGSaveOptions();
                        with (jpgOpt)
                            {
                                embedColorProfile = false;
                                matte = MatteType.WHITE;
                                quality = 6;
                                formatOptions = FormatOptions.STANDARDBASELINE;
                            }
                        documentSelection.saveAs(new File(sPath + nameFolderPubReady + fileNamePubReadyImage), jpgOpt, true, Extension.LOWERCASE);
                        bc.closeDocumentNoSave(bc.documentOriginal);
                        bc.closeDocumentNoSave(documentSelection);
                    }
                else
                    {
                        // write to the exception file
                        var fExceptions = new File(sPath + nameFolderPubReady + "exceptions.txt");
                        if (fExceptions.exists) fExceptions.open("a");
                        else fExceptions.open("w");
                        with (fExceptions)
                            {
                                writeln(asFiles[i].name + " reports:");
                                writeln(exception);
                                close();
                            }
                    }
            } 
    }