﻿/*
    
    Since once a script is running there can be no interaction with Photoshop, the user must make a selection before the script
    is fired.  The BubbleCounter class will look to see if there is a selection and execution will halt if there is not.
    
    User selects an area of the image representative of the white balance card so that a bimodal distribution of pixel intensities
    approximating black and white is generated.
    
*/



// set the debug level appropriately, remark if editing code
$.level = 0;

//  set the target application to photoshop
#target photoshop

//  include the BubbleCounter script or none of this beautiful work will
#include "BC_Classes/BubbleCounter.jsxinc"

//  turn off dialogs, seems to speed processing?
app.displayDialogs = DialogModes.NO;
//  get the present ruler units so that the app can eb set back to its original state
var uStrtRulerUnits = app.preferences.rulerUnits;
//  set the ruler units to pixels
app.preferences.rulerUnits = Units.PIXELS;

// make sure a document is open
if (app.documents.length > 0)
    {
        //  instantiate a BubbleCounter class
        var bc = new BubbleCounter();

        //  set the documentOriginal property to the current active document
        bc.documentOriginal = app.activeDocument;
         // make sure that something has been selected and it appears to be a reasonable size
        if (bc.isSelectionMade())
            {
                    if (bc.runWhiteBalance())
                        {
                            alert("The white balance has been determined.  Select the area to analyze and run BC Analyze to analyze the currently open image.");
                        }
                    else
                        {
                            alert("The white balance script exited without calculating a range.  An unknown error occured.");
                        }
            }
        else
            {
                alert("Please select an area representative of the contents of the white balance card and rerun the script.");
            }
    }
 else
    {
        alert("Please open an image of a scanned, prepared slab and rerun this script.")
    }

app.preferences.rulerUnits = uStrtRulerUnits;